--[[--

游戏主文件

]]

function __G__TRACKBACK__(msg)
    local errorMsg = ""
    local pid = ""

    if UserManager and UserManager.getPID then 
        pid = UserManager:getPID()
        if pid == -1 then pid = "" end 
    end 

    errorMsg = errorMsg .. "LUA ERROR: (" .. pid .. ")" .. tostring(msg) .. "\n"
    errorMsg = errorMsg .. debug.traceback()

    --request  
    if not DEBUG_ENV and network and network.createHTTPRequest then 
        local url = "https://oapi.dingtalk.com/robot/send?access_token=b68d961677b6f7c18a312fec0969a1026e7a8a4b7a82a5d902559344aed36a2c"
        local req = network.createHTTPRequest( function(event) end, url, "POST" )
        local postJson = json.encode({ msgtype = "text", text = { content = errorMsg }, })
        req:setPOSTData(postJson)
        req:addRequestHeader("Content-Type:application/json")
        req:start()
    end 

    print("----------------------------------------")
    print(errorMsg)
    print(debug.traceback())
    print("----------------------------------------")
end

require "lfs"
local fu = CCFileUtils:sharedFileUtils()

local function exists(path)
    return fu:isFileExist(path)
end 

local function writeFile(path, content, mode)
    mode = mode or "w+b"
    local file = io.open(path, mode)
    if file then
        if file:write(content) == nil then return false end
        io.close(file)
        return true
    else
        return false
    end
end

--移除文件夹
local function rmdir(path)
    if exists(path) then
        local function _rmdir(path)
            local iter, dir_obj = lfs.dir(path)

            while true do
                local dir = iter(dir_obj)
                if dir == nil then break end
                if dir ~= "." and dir ~= ".." then
                    local curDir = path..dir
                    local mode = lfs.attributes(curDir, "mode")
                    if mode == "directory" then
                        _rmdir(curDir.."/")
                    elseif mode == "file" then
                        os.remove(curDir)
                    end
                end
            end

            local succ, des = os.remove(path)
            if des then print(des) end
            return succ
        end

        _rmdir(path)
    end
    return true
end

local function needCleanCache()
    print("#### 判定是否需要清除缓存资源 ####")
    local ures = fu:getWritablePath().."Resources/"
    local buildFile = fu:getWritablePath().."build.txt"
    local isExistCacheFile = exists(ures.."version.txt")
    print("#### isExistCacheFile? ", isExistCacheFile)
    if not _G_BUILD_NUM_ then _G_BUILD_NUM_ = 0 end

    --判定是否存在Rescources/version.txt 若存在，直接对比; 若不存在，则先判断是否存在缓存文件夹，存在则需要清空(防止用户主动删除version文件)
    if not isExistCacheFile then 
        if not exists(buildFile) then 
            writeFile(buildFile,_G_BUILD_NUM_)
        end 

        if exists(ures.."wol") then 
            print("#### 若不存在，则先判断是否存在缓存文件夹 ####")
            return true
        end 

        return false 
    else 
        local cacheInfo = fu:getFileData(ures.."version.txt")
        local resInfo   = fu:getFileData("res/version.txt")
        print("#### 存在，cacheInfo =", cacheInfo, ", resInfo =", resInfo)
        local cacheBigVersion = string.gsub(string.sub(cacheInfo, string.find(cacheInfo, "%b..")), "%.", "")
        local resBigVersion   = string.gsub(string.sub(resInfo, string.find(resInfo, "%b..")), "%.", "")
        print("#### 对比：resBigVersion =", resBigVersion, ", cacheBigVersion =", cacheBigVersion)
        
        if tonumber(resBigVersion) > tonumber(cacheBigVersion) then 
            return true 
        end 

        if not exists(buildFile) then 
            writeFile(buildFile,_G_BUILD_NUM_)
            return true 
        else 
            local num = tonumber(fu:getFileData(buildFile) or 0)
            if num < _G_BUILD_NUM_ then 
                writeFile(buildFile,_G_BUILD_NUM_)
                return true
            end 
        end
    end 

    return false 
end 

local function cleanCache()
    local fu = CCFileUtils:sharedFileUtils()
    local ures = fu:getWritablePath().."Resources/"

    print("#### 确实存在缓存，且需要清理，执行清理工作 ####")

    if exists(ures) then rmdir(ures) end 

    _VERSION_NUMBER_ = nil 
end 

--[[--

main函数，游戏入口

]]

local function main()
    --防止内存泄露
    collectgarbage("setpause", 100)
    collectgarbage("setstepmul", 5000)
    
    --初始化脚本搜索路径    
    device = device or {}
    device.writablePath = fu:getWritablePath()
    fu:addSearchPath(device.writablePath.."Resources/wol/scripts/")
    fu:addSearchPath(device.writablePath.."Resources/wol/res/ccs/")
    fu:addSearchPath(device.writablePath.."Resources/wol/res/")
    if _LUAGIT_VERSION_ == 64 then 
        fu:addSearchPath("scripts64/")
    else 
        fu:addSearchPath("scripts/")
    end 
    fu:addSearchPath("res/ccs/")
    fu:addSearchPath("res/")

    --增加让缓存区生效的代码
    local files = {}
    local scriptsPath = device.writablePath.."Resources/wol/scripts"

    --优先判定是否大版本更新，是则需要清除缓存数据
    if needCleanCache() then cleanCache() end 

    -- print("******** 增加让缓存区生效的代码 **********")

    --必须优先判断该文件夹存在
    if exists(scriptsPath) then 

        local function removePreloadFromPath(modelPath)
            -- print("remove : ", modelPath)

            if not modelPath or string.len(modelPath) == 0 then
                return
            end

            local modname = string.gsub(modelPath, "%.lua", "")
            modname = string.gsub(modname, "/", "%.")
            -- print("modname : ", modname)

            if modname and package.preload[modname] then 
                -- print("****** find modname = ", modname)
                package.preload[modname] = nil
            end 
        end 

        local function _addFile(path)
            local iter, dir_obj = lfs.dir(path)

            while true do
                local dir = iter(dir_obj)
                if dir == nil then break end

                if dir ~= "." and dir ~= ".." and dir ~= ".DS_Store" then
                    local curDir = path.."/"..dir

                    local mode = lfs.attributes(curDir, "mode")

                    if mode == "directory" then
                        _addFile(curDir)

                    elseif mode == "file" then
                        --需要处理一下特殊字符[-]
                        local _curDir = (string.gsub(curDir, "[-]", ""))
                        local ocapy = (string.gsub(scriptsPath, "[-]", ""))
                        local scriptpath = (string.gsub(_curDir, ocapy.."/", ""))
                        
                        removePreloadFromPath(scriptpath)
                    end
                end
            end
        end 
        
        _addFile(scriptsPath)
    end 

    --加载lua文件
    require "lib.Import"

    import "lib.core.S"
    import "lib.core.IO"
    import "lib.core.App"
    import "lib.core.AppEvent" 
    import "lib.core.AppNet"
    import "lib.core.ProtoBuff"
    import "lib.core.Element"
    import "lib.core.Net"
    import "lib.core.Packet"
    import "lib.core.Quad"
    import "lib.core.Box"
    import "lib.core.SocketNet"
    import "lib.core.Stage"
    import "lib.core.Main"

    --优先加载必要的库，更新后需清理重新加载
    require "config"
    require "framework.init"
    require "framework.shortcodes"
    require "framework.cc.init" 
    require "framework.cc.ui.init"

    --for anaylitics
    require "dp.Anaylitics"
    require "common.Utils"
    require "lib.lang.lang"
    require "lib.lang.lang2"
    require "lib.lang.lang3"

    --重写CCS label组件的getContentSize方法
    local label = Label
    label.getContentSize = function(label)
        local size = label:getBoundingBoxSize()
        -- if label:isIgnoreContentAdaptWithSize() then
        --     return size
        -- else
            return CCSize(size.width/2, size.height/2)
        -- end
    end
    Label = label

    --加载闪屏
    local loadScene = require "loader.LoadScene"
    CCDirector:sharedDirector():runWithScene(loadScene)
    
end

xpcall(main, __G__TRACKBACK__)